-- Most of the idea to the code (and the "well written" part of it :-) are by Phanx, whom I thanks for the helps and patience in explaining. 

local ADDON = ...

AGGROMON_ACTIVE = AGGROMON_ACTIVE or 1
AGGROMON_SOUND = AGGROMON_SOUND or 1
AGGROMON_CHAT = AGGROMON_CHAT or 0
AGGROMON_SHOW = AGGROMON_SHOW or 1
AGGROMON_SNDFLE = AGGROMON_SNDFLE or 1
AGGROMON_ENAPVP = AGGROMON_ENAPVP or 1

local prgname = "|cffffd200aggromon|r"
local string_format = string.format
local aggromon_status, aggromon_status_old, aggromon_value, aggromon_summary

-- Frame related vars
local bgfile = "Interface\\ChatFrame\\ChatFrameBackground"
local edgefile = "Interface\\Tooltips\\UI-Tooltip-Border"
local framefontn, framefonth, framefontf = GameFontNormal:GetFont()

-- Define all of the locals (functions) first, and then set their values in whatever order
local Aggromon_print_summary, Aggromon_check
local frame_cleu, frame_cleu2, frame_al
local Aggromon_AggroFrame, Aggromon_AggroValue
local aggromon_lvl, aggromon_color, aggromon_soundfile

local tostring = tostring

aggromon_lvl = {
	"in aggro, but not tanking anything",
	"not tanking anything, but have higher threat than tank on at least one unit",
	"insecurely tanking at least one unit, but not securely tanking anything",
	"securely tanking at least one unit",
}
	-- nil = unit is not on any other unit's threat table.
	-- 0 = not tanking anything.
	-- 1 = not tanking anything, but have higher threat than tank on at least one unit.
	-- 2 = insecurely tanking at least one unit, but not securely tanking anything.
	-- 3 = securely tanking at least one unit. 
	-- descriptions taken from: http://www.wowwiki.com/API_UnitThreatSituation

	-- nil  
	-- 0 = Unit has less than 100% raw threat (default UI shows no indicator)
    -- 1 = Unit has 100% or higher raw threat but isn't mobUnit's primary target (default UI shows yellow indicator)
    -- 2 = Unit is mobUnit's primary target, and another unit has 100% or higher raw threat (default UI shows orange indicator)
    -- 3 = Unit is mobUnit's primary target, and no other unit has 100% or higher raw threat (default UI shows red indicator)
	-- descriptions taken from: http://wowprogramming.com/docs/api/UnitThreatSituation
	
aggromon_color = {
	{1,1,0},
	{1,0.5,0},
	{1,0.2,0},
	{1,0,0},
}
	
aggromon_soundfile = {
	"Interface\\AddOns\\Aggromon\\sounds\\babe.ogg",
	"Interface\\AddOns\\Aggromon\\sounds\\zenham.ogg",
	"Sound\\Doodad\\BellTollAlliance.wav",
	"Sound\\interface\\AuctionWindowClose.wav",
	
	-- Sounds taken from banzai-alert: http://www.wowace.com/addons/banzai-alert/ 
}

function Aggromon_check()

	local disable = 0

	if AGGROMON_ACTIVE == 0 then 
		disable = 1 
	end
	
	local _, instanceType = IsInInstance()
	if AGGROMON_ENAPVP == 0 and (instanceType == "arena" or instanceType == "pvp" or (instanceType == "none" and GetZonePVPInfo() == "combat")) then
		disable = 1
	end

	if disable == 1 then
		Aggromon_AggroFrame:Hide()
		frame_cleu:UnregisterEvent("UNIT_THREAT_SITUATION_UPDATE")
		frame_cleu2:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		frame_cleu2:UnregisterEvent("UNIT_THREAT_SITUATION_UPDATE")
	else
		Aggromon_AggroFrame:Show()
		frame_cleu:RegisterEvent("UNIT_THREAT_SITUATION_UPDATE")
		frame_cleu2:RegisterEvent("UNIT_THREAT_SITUATION_UPDATE")
		frame_cleu2:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	end
		
	return		
end
	
function Aggromon_print_summary ()
		if AGGROMON_SHOW == 0 then	
			aggromon_summary = "[show no] "
		else
			aggromon_summary = "[show yes] "
		end
		
		if AGGROMON_CHAT == 0 then	
			aggromon_summary = aggromon_summary .. "[print no] "
		else
			aggromon_summary = aggromon_summary .. "[print yes] "
		end
		
		if AGGROMON_SOUND == 0 then	
			aggromon_summary = aggromon_summary .. "[sound no] "
		else
			aggromon_summary = aggromon_summary .. "[sound yes] "
		end

		if AGGROMON_ENAPVP == 0 then	
			aggromon_summary = aggromon_summary .. "[pvp no] "
		else
			aggromon_summary = aggromon_summary .. "[pvp yes] "
		end
		
		if AGGROMON_ACTIVE == 0 then			
			print(string_format("%s: |cffff0000disabled|r. Type /%s to enable", prgname, prgname))
		else
			print(string_format("%s: |cff228B22enabled|r %s" , prgname, aggromon_summary))
		end 
end
		
		
-- Frame colored based on aggro
Aggromon_AggroFrame = CreateFrame("Frame", ADDON.."_Show", UIParent)
Aggromon_AggroFrame:SetBackdrop({bgFile = bgfile, edgeFile = edgefile, edgeSize = 18, insets = {left = 5, right = 5, top = 5, bottom = 5}})
Aggromon_AggroFrame:SetBackdropColor(0,0,0,0.8)
Aggromon_AggroFrame:SetWidth(40)
Aggromon_AggroFrame:SetHeight(40)
Aggromon_AggroFrame:SetPoint("CENTER",UIParent)
Aggromon_AggroFrame:EnableMouse(true)
Aggromon_AggroFrame:SetMovable(true)
Aggromon_AggroFrame:RegisterForDrag("LeftButton")
Aggromon_AggroFrame:SetScript("OnDragStart", function(self) self:StartMoving() end)
Aggromon_AggroFrame:SetScript("OnDragStop", function(self) self:StopMovingOrSizing() end)		
Aggromon_AggroFrame:SetClampedToScreen(true)

	
Aggromon_AggroValue = Aggromon_AggroFrame:CreateFontString("Aggromon_AggroFrame")
Aggromon_AggroValue:SetPoint("CENTER", Aggromon_AggroFrame,"CENTER",0, 0)
Aggromon_AggroValue:SetFont(GameFontNormal:GetFont(),(framefonth))
	
	
-- Frame to register the global vars at the ADDON_LOADED event.	
frame_al = CreateFrame("FRAME", ADDON.."_al")
frame_al:RegisterEvent("ADDON_LOADED")	
frame_al:SetScript("OnEvent", function(self, event, arg1, ...)	
	if event == "ADDON_LOADED" and arg1 == "Aggromon" then			
		if AGGROMON_SHOW == 1 then 
			Aggromon_AggroFrame:Show()
		else
			Aggromon_AggroFrame:Hide()
		end		
		Aggromon_print_summary()		
		self:UnregisterEvent("ADDON_LOADED")		
	end	
end)

-- Frame to parse UNIT_THREAT_SITUATION_UPDATE	
frame_cleu = CreateFrame("FRAME", ADDON.."_cleu")
frame_cleu:RegisterEvent("ZONE_CHANGED_NEW_AREA")
frame_cleu:RegisterEvent("PLAYER_ENTERING_WORLD")
frame_cleu:SetScript("OnEvent", function(self, event, ...)

	if event == "ZONE_CHANGED_NEW_AREA" or event == "PLAYER_ENTERING_WORLD" then
		Aggromon_check()
	end 
	
	aggromon_status = UnitThreatSituation("player")
	
	-- show the colored frame
	if AGGROMON_SHOW == 1 then	
		if aggromon_status then
			Aggromon_AggroFrame:SetBackdropColor(aggromon_color[aggromon_status+1][1],aggromon_color[aggromon_status+1][2],aggromon_color[aggromon_status+1][3],0.8)
		else
			Aggromon_AggroFrame:SetBackdropColor(0,1,0,0.8)			
		end					
	end 	
	-- print on the console
	if AGGROMON_CHAT == 1 then 
		if aggromon_status then 
			DEFAULT_CHAT_FRAME:AddMessage(string_format("aggromon: [threat lvl:%i] [target:%s] %s", aggromon_status, aggromon_value, aggromon_lvl[aggromon_status+1]), aggromon_color[aggromon_status+1][1],aggromon_color[aggromon_status+1][2],aggromon_color[aggromon_status+1][3]) 
		else  
			DEFAULT_CHAT_FRAME:AddMessage("aggromon: [threat nil] no aggro, unit is not on any other unit's threat table.",0,1,0) 
		end
	end 
	-- play the old "Aggro!" ... It miss me so much ...  ! :-)
	if AGGROMON_SOUND == 1 then 
		if aggromon_status and aggromon_status >= 1 and aggromon_status_old == nil then
			PlaySoundFile(aggromon_soundfile[AGGROMON_SNDFLE], "Master") 	
		end
	end 		
	-- store the actual aggro level
	aggromon_status_old = aggromon_status
end)
	

-- create a second frame to register also the COMBAT_LOG to update the value aggromon_value	
frame_cleu2 = CreateFrame("FRAME", ADDON.."_cleu2")
frame_cleu2:RegisterEvent("ZONE_CHANGED_NEW_AREA")
frame_cleu2:RegisterEvent("PLAYER_ENTERING_WORLD")
frame_cleu2:SetScript("OnEvent", function(self, event, ...)

	if event == "ZONE_CHANGED_NEW_AREA" or event == "PLAYER_ENTERING_WORLD" then
		Aggromon_check()
	end 

	_,_,aggromon_value,_,_ = UnitDetailedThreatSituation("player", "target");
	aggromon_value = tostring(string_format("%.0f",(aggromon_value or 0)))
	Aggromon_AggroValue:SetText(aggromon_value)

end)

-- Configuration Panel -------------------------------------------------------------------------------------

local options = CreateFrame("Frame", ADDON.."Options", InterfaceOptionsFramePanelContainer)
options.name = GetAddOnMetadata(ADDON, "Title") or ADDON
InterfaceOptions_AddCategory(options)

local title = options:CreateFontString("$parentTitle", "ARTWORK", "GameFontNormalLarge")
title:SetPoint("TOPLEFT", 16, -16)
title:SetText(options.name)
options.title = title

local subtext = options:CreateFontString("$parentSubText", "ARTWORK", "GameFontHighlightSmall")
subtext:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 0, -8)
subtext:SetPoint("RIGHT", -32, 0)
subtext:SetHeight(32)
subtext:SetJustifyH("LEFT")
subtext:SetJustifyV("TOP")
subtext:SetText(GetAddOnMetadata(ADDON, "Notes"))
options.subtext = subtext

local enable = CreateFrame("CheckButton", "$parentEnable", options, "InterfaceOptionsCheckButtonTemplate")
enable:SetPoint("TOPLEFT", subtext, "BOTTOMLEFT", -2, -8)
enable.Text:SetText("Enable")
enable.tooltipText = "Enable global monitoring"
enable:SetChecked(AGGROMON_ACTIVE);
enable:SetScript("OnClick", function(self)
	if enable:GetChecked() then 
		AGGROMON_ACTIVE = 1
		print(string_format("%s is enabled",prgname))
		Aggromon_check()
	else 
		AGGROMON_ACTIVE = 0
		print(string_format("%s is disabled",prgname))
		Aggromon_check()
	end	
end)

local enable_pvp = CreateFrame("CheckButton", "$parentEnable", options, "InterfaceOptionsCheckButtonTemplate")
enable_pvp:SetPoint("TOPLEFT", subtext, "BOTTOMLEFT", -2, -38)
enable_pvp.Text:SetText("PVP zones")
enable_pvp.tooltipText = "Enable monitoring in PVP zones"
enable_pvp:SetChecked(AGGROMON_ENAPVP);
enable_pvp:SetScript("OnClick", function(self)
	if enable_pvp:GetChecked() then 
		AGGROMON_ENAPVP = 1
		print(string_format("%s is enabled in pvp zones",prgname))
		Aggromon_check()
	else 
		AGGROMON_ENAPVP = 0
		print(string_format("%s is disabled in pvp zones",prgname))
		Aggromon_check()
	end	
end)


local sampletext = options:CreateFontString("$parentSampleText", "ARTWORK", "GameFontHighlight")
sampletext:SetPoint("TOPLEFT", subtext, "BOTTOMLEFT", -2, -120)
sampletext:SetText('Alerts:')

local chatout = CreateFrame("CheckButton", "$parentCHATOUT", options, "InterfaceOptionsCheckButtonTemplate")
chatout:SetPoint("TOPLEFT", subtext, "BOTTOMLEFT", -2, -140)
chatout.Text:SetText("Chat (spammy)")
chatout.tooltipText = "Print on chat"
chatout:SetChecked(AGGROMON_CHAT);
chatout:SetScript("OnClick", function(self)
	if chatout:GetChecked() then 
		AGGROMON_CHAT = 1
		print(string_format("%s: output on chat is enabled",prgname))
	else 
		AGGROMON_CHAT = 0
		print(string_format("%s: output on chat is disabled",prgname))
	end	
end)	

local legenda = CreateFrame("button", "$parentLegenda", options, "UIPanelButtonTemplate")
legenda:SetHeight(25)
legenda:SetWidth(80)
legenda:SetPoint("TOPLEFT", subtext, "BOTTOMLEFT", 170, -140)
legenda:SetText("Legenda")
legenda:SetScript("OnClick", 
	function()
		local index
		print(prgname .. " colors:")
		DEFAULT_CHAT_FRAME:AddMessage("aggromon: [threat nil] no aggro, unit is not on any other unit's threat table.",0,1,0) 
		for index = 0, 3, 1 do
			DEFAULT_CHAT_FRAME:AddMessage(string_format("aggromon: [threat lvl:%i] %s", index, aggromon_lvl[index+1]), aggromon_color[index+1][1],aggromon_color[index+1][2],aggromon_color[index+1][3]) 
		end
	end) 

local frameshow = CreateFrame("CheckButton", "$parentFRAMESHOW", options, "InterfaceOptionsCheckButtonTemplate")
frameshow:SetPoint("TOPLEFT", subtext, "BOTTOMLEFT", -2, -170)
frameshow.Text:SetText("Aggro Frame")
frameshow.tooltipText = "Show the little aggro frame"
frameshow:SetChecked(AGGROMON_SHOW);
frameshow:SetScript("OnClick", function(self)
	
	if frameshow:GetChecked() then 
		AGGROMON_SHOW = 1
		print(string_format("%s: aggro frame is enabled",prgname))
		Aggromon_AggroFrame:Show()
	else 
		AGGROMON_SHOW = 0
		print(string_format("%s: aggro frame is disabled",prgname))
		Aggromon_AggroFrame:Hide()
	end	
	
end)	

local sound = CreateFrame("CheckButton", "$parentSND", options, "InterfaceOptionsCheckButtonTemplate")
sound:SetPoint("TOPLEFT", subtext, "BOTTOMLEFT", -2, -200)
sound.Text:SetText("Sound Alert")
sound.tooltipText = "Sound"
sound:SetChecked(AGGROMON_SOUND);
sound:SetScript("OnClick", function(self)
	
	if sound:GetChecked() then 
		AGGROMON_SOUND = 1
		print(string_format("%s: sound alert is enabled",prgname))
	else 
		AGGROMON_SOUND = 0
		print(string_format("%s: sound alert is disabled",prgname))
	end	
	
end)		

local testsnd = CreateFrame("button", "$parentTestSound", options, "UIPanelButtonTemplate")
testsnd:SetHeight(25)
testsnd:SetWidth(80)
testsnd:SetPoint("TOPLEFT", subtext, "BOTTOMLEFT", 170, -200)
testsnd:SetText("Play it")
testsnd:SetScript("OnClick", 
	function()
		PlaySoundFile(aggromon_soundfile[AGGROMON_SNDFLE], "Master") 
	end) 
	
local sndfile = CreateFrame("Slider", "$parentSNDFILE", options, "OptionsSliderTemplate")
sndfile:SetPoint("TOPLEFT", enable, "BOTTOMLEFT", 6, -230)
sndfile:SetWidth(250)
sndfile:SetMinMaxValues(1, 4)
sndfile:SetValue(1)
sndfile.low = _G[sndfile:GetName().."Low"]
sndfile.low:SetPoint("TOPLEFT", sndfile, "BOTTOMLEFT", 0, 0)
sndfile.low:SetText('1')
sndfile.high = _G[sndfile:GetName().."High"]
sndfile.high:SetPoint("TOPRIGHT", sndfile, "BOTTOMRIGHT", 0, 0)
sndfile.high:SetText('4')
sndfile.text = _G[sndfile:GetName().."Text"]
sndfile.text:SetText("Sounds")
sndfile.tooltipText = "Sound"
sndfile.value = sndfile:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
sndfile.value:SetPoint("TOP", sndfile, "BOTTOM", 0, 0)
sndfile.value:SetText(AGGROMON_SNDFLE)
sndfile:SetScript("OnValueChanged", function(self, value)
	AGGROMON_SNDFLE = floor(value + 0.5)
	self:SetValue(AGGROMON_SNDFLE)
	self.value:SetText(AGGROMON_SNDFLE)
end)

function options.refresh()
	enable:SetChecked(AGGROMON_ACTIVE)

end

if LibStub and LibStub("LibAboutPanel", true) then
	options.about = LibStub("LibAboutPanel").new(options.name, ADDON)
end

-- CMD Parse --------------------------------------------------------------------------------------------
	
SLASH_AGGROMON1 = "/aggromon";
SlashCmdList["AGGROMON"] = function() 
	InterfaceOptionsFrame_OpenToCategory(options)
end